----------------------------------------------------------------------------------
-- SPI_mgr_V2
-- Version 2.00 - rewritten from SPI_mgr_V1 by YB (2015/09/03)
--    - Removed options that were specific to the Spartan 3E/3A/3AN starter kit
--    - Added a wider prescaler
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity OLED_SPI_mgr is
	 Generic( RXTX_ADDR : std_logic_vector(8 downto 0);
				 ST_ADDR   : std_logic_vector(8 downto 0));
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
			  pic_out : in  STD_LOGIC_VECTOR (7 downto 0);
			  pic_in  : out  STD_LOGIC_VECTOR (7 downto 0);
			  addr : in  STD_LOGIC_VECTOR (8 downto 0);
			  pic_w : in  STD_LOGIC;
			  SPI_SCK  : out std_logic;
			  SPI_MOSI : out std_logic;
			  SS_0     : out std_logic);
end OLED_SPI_mgr;

--
-- status bits (write only): 
--		7-2 : prescaler (uses clk / 2(n+1) for SPI_SCK)  
--		1   : SS1
--		0   : SS0
--
-- status bits (read only): 
--		7-1 : read as 0 / reserved for future use
--		0   : busy



architecture Behavioral of OLED_SPI_mgr is

signal slave_selects : std_logic_vector(0 downto 0);
signal busy          : std_logic;
signal mosi_buffer   : std_logic_vector(7 downto 0);
signal presc_buf     : std_logic_vector(9 downto 0);
signal presc_val     : std_logic_vector(9 downto 0);
signal bit_count     : std_logic_vector(3 downto 0);
signal clk_out       : std_logic;

begin

SPI_SCK  <= clk_out;
SS_0     <= not ( slave_selects(0) );

process(clk)
begin
  if clk'event and clk='1' then
	  if reset = '1' then
			SPI_MOSI <= '1';
	  elsif clk_out='1' and presc_val = "0000000000" and bit_count/="0000" then
			SPI_MOSI <= mosi_buffer(7);
	  elsif bit_count="0000" and pic_w='1' and addr=RXTX_ADDR then
	      SPI_MOSI <= pic_out(7);
	  end if;
  end if;
end process;

process(clk)
begin
  if clk'event and clk='1' then
	  if reset = '1' then
			mosi_buffer <= "00000000";
	  elsif bit_count/="0000" and clk_out='1' and presc_buf = presc_val then
			mosi_buffer <= mosi_buffer(6 downto 0) & '0';
	  elsif bit_count="0000" and pic_w='1' and addr=RXTX_ADDR then
	      mosi_buffer <= pic_out;
	  end if;
  end if;
end process;

process(clk)
begin
  if clk'event and clk='1' then
	  if reset = '1' or bit_count="0000" then
			busy <= '0';
	  else
	      busy <= '1';
	  end if;
  end if;
end process;

process(clk)
begin
  if clk'event and clk='1' then
	  if reset = '1' then
			presc_buf     <= "0000000000";
			slave_selects <= "0";		
	  elsif pic_w='1' and addr=ST_ADDR then
         case pic_out(4 downto 2) is
            when "000" => presc_buf <= "0000000" & pic_out(7 downto 5);
            when "001" => presc_buf <= "000000" & pic_out(7 downto 5) & "1";
            when "010" => presc_buf <= "00000" & pic_out(7 downto 5) & "1" & pic_out(7);
            when "011" => presc_buf <= "0000" & pic_out(7 downto 5) & "1" & pic_out(7 downto 6);
            when "100" => presc_buf <= "000" & pic_out(7 downto 5) & "1" & pic_out(7 downto 5);
            when "101" => presc_buf <= "00" & pic_out(7 downto 5) & "1" & pic_out(7 downto 5) & "1";
            when "110" => presc_buf <= "0" & pic_out(7 downto 5) & "1" & pic_out(7 downto 5) & "1" & pic_out(7);
            when others => presc_buf <=     pic_out(7 downto 5) & "1" & pic_out(7 downto 5) & "1" & pic_out(7 downto 6);
         end case;
			slave_selects <= pic_out(0 downto 0);
	  end if;
  end if;
end process;

process(clk)
begin
	if clk'event and clk='1' then
	   if reset='1' then
			clk_out   <= '0';
			presc_val <= "0000000000";
		elsif presc_val = "0000000000" then
				if bit_count/= "0000" then
					clk_out   <= not clk_out;
					presc_val <= presc_buf;
				else
					clk_out   <= '0';
					if pic_w='1' and addr=RXTX_ADDR then
						Presc_val <= presc_buf;
					end if;
				end if;
		else
				presc_val <= presc_val - 1;
		end if;
	end if;
end process;


process(clk)
begin
  if clk'event and clk='1' then
	  if reset = '1' then
			bit_count <= "0000";
	  elsif bit_count/="0000" then
			if presc_val = "0000000000" and clk_out = '1' then
				bit_count <= bit_count - 1;
			end if;
	  elsif pic_w='1' and addr=RXTX_ADDR then
			bit_count <= "1000";
	  end if;
  end if;
end process;

process(addr, busy, mosi_buffer)
begin
   if addr=ST_ADDR  then
		pic_in <= "0000000" & busy;
	elsif addr=RXTX_ADDR then
		pic_in <= mosi_buffer;
	else
		pic_in <= "00000000";
	end if;
end process;





end Behavioral;

