-- Toplevel for the OLED display controler
-- (C) 2015/10/21 - Yannick Bornat - ENSEIRB-MATMECA

library ieee;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;

entity OLED_ctrl is
   generic ( reverse_display : std_logic:='0');
   port ( clk     	: in  std_logic; 
          resetn     : in  std_logic;
          OLED_CS    : out std_logic;
          OLED_MOSI  : out std_logic;
          OLED_SCLK  : out std_logic;
          OLED_D_nC  : out std_logic;
          OLED_RES   : out std_logic;
          OLED_VBATC : out std_logic;
          OLED_VDDC  : out std_logic;
          char_in    : in  std_logic_vector(7 downto 0);
          col        : in  std_logic_vector(3 downto 0);
          line       : in  std_logic_vector(1 downto 0);
          char_write : in  std_logic;
          write_ack  : out std_logic);
			 
			 
end OLED_ctrl;

architecture BEHAVIORAL of OLED_ctrl is

--   constant clk100M : integer :=1;

   signal ctrl_gathered : std_logic_vector (7 downto 0);

   signal loc_bus_dout  : std_logic_vector (7 downto 0);
   signal loc_bus_addr  : std_logic_vector (8 downto 0);
   signal pc            : std_logic_vector (10 downto 0);
   signal reset         : std_logic;
   signal opcode_pic_in : std_logic_vector (13 downto 0);
   signal code_ram_out  : std_logic_vector (13 downto 0);
   signal loc_bus_write : std_logic;
   signal code_ram_dout : std_logic_vector (15 downto 0);
   signal code_ram_din  : std_logic_vector (15 downto 0);
   signal code_ram_addr : std_logic_vector (10 downto 0);
   signal code_ram_wr   : std_logic;
   signal oled_spi_out  : std_logic_vector (7 downto 0);
   signal coderam_lcout : std_logic_vector (7 downto 0);
   signal loc_bus_din   : std_logic_vector (7 downto 0);
   signal loc_bus_read  : std_logic;

   signal io_ctrl        : std_logic_vector(7 downto 4);

   

begin

   ctrl_gathered <= char_write & reverse_display & line & col;
   
   OLED_D_nC  <= io_ctrl(4);
   OLED_RES   <= io_ctrl(5);
   OLED_VBATC <= io_ctrl(6);
   OLED_VDDC  <= io_ctrl(7);
   
   reset     <= not resetn;

   CPU : entity work.OLED_CPU
      port map (Clk           => clk,
                code_data     => opcode_pic_in,
                Reset_n       => resetn,
                code_addr     => pc,
                ext_data_addr => loc_bus_addr,
                ext_data_out  => loc_bus_dout,
                ext_re        => loc_bus_read,
                ext_we        => loc_bus_write,
                ext_data_in   => loc_bus_din);



   process(loc_bus_addr, char_in, ctrl_gathered, coderam_lcout, oled_spi_out, io_ctrl)
   begin
      case loc_bus_addr is
         when "000000101" => loc_bus_din <= char_in;
         when "000000110" => loc_bus_din <= ctrl_gathered;
         when "011000000" => loc_bus_din <= io_ctrl & "0000";
         when others      => loc_bus_din <= coderam_lcout or oled_spi_out;
      end case;
   end process;
   
   
   process(clk)
   begin
      if rising_edge(clk) then
         if loc_bus_addr = "000000110" and loc_bus_write = '1' then
            write_ack <= loc_bus_dout(0);
         end if;
         if loc_bus_addr = "011000000" and loc_bus_write = '1' then
            io_ctrl <= loc_bus_dout(7 downto 4);
         end if;
      end if;
   end process;

   OLED_SPI : entity work.OLED_SPI_mgr
		generic map (RXTX_addr   => "000010111", --0x017
						 ST_addr     => "000010110") --0x016
      port map (clk           => clk,
                reset         => reset,
                pic_out       => loc_bus_dout,
                pic_in        => oled_spi_out,
                addr          => loc_bus_addr,
                pic_w         => loc_bus_write,
                SPI_SCK       => OLED_SCLK,
                SPI_MOSI      => OLED_MOSI,
                SS_0          => OLED_CS);


                
opcode_pic_in <= code_ram_out;

   
   Coderam_mgr : entity work.OLED_RAM_mgr
	   generic map (ADDL =>		"010001100", 		-- 0x08C
						 ADDH =>		"010001101", 		-- 0x08D
						 DATL =>		"010001110", 		-- 0x08E
						 DATH =>		"010001111") 		-- 0x08F
      port map (reset         => reset,
					 clk           => clk,
                pic_out       => loc_bus_dout,
                pic_w         => loc_bus_write,
                addr          => loc_bus_addr,
                ram_din       => code_ram_dout,
                pic_in        => coderam_lcout,
                ram_addr      => code_ram_addr,
                ram_dout      => code_ram_din,
                ram_write     => code_ram_wr);


   
   RAM_prog : entity work.OLED_def_RAM_10
      port map (addra         => pc,
                addrb         => code_ram_addr,
                clk           => clk,
                dinb          => code_ram_din,
                w_b           => code_ram_wr,
                douta         => code_ram_out,
                doutb         => code_ram_dout);



   
end BEHAVIORAL;


